{% if cruditor.titlebuttons %}
	<div class="title-buttons">
		{% for button in cruditor.titlebuttons %}
			<a href="{{ button.url }}" class="btn btn-{{ button.button_class|default:'light' }}">{{ button.label }}</a>
		{% endfor %}
	</div>
{% endif %}
