{% extends 'cruditor/base.html' %}
{% load i18n %}


{% block breadcrumb %}
{% endblock %}

{% block container %}
<div class="jumbotron">
		<h1 class="display-4">{% trans 'Not found' %}</h1>
		<p class="lead">{% trans 'The requested item could not be found' %}</p>
		<hr class="my-4">
		<p class="text-right">
			<a href="{{ cruditor.constants.index_url }}" class="btn btn-primary">
				{% trans 'Take me to the index page' %}
			</a>
		</p>
	</div>
</div>
{% endblock %}
