{% load static %}


<!DOCTYPE html>
<html>
	<head>
		<title>{{ cruditor.title }}</title>
		<meta charset="utf-8">
		<meta name="viewport" content="width=device-width, initial-scale=1">

		{% block styles %}
			<link rel="stylesheet" href="{% static 'cruditor/css/cruditor.css' %}" />
		{% endblock %}

		{% block extrahead %}
			{% include cruditor.constants.extrahead_template_name %}
		{% endblock %}
	</head>
	<body class="{% block body_class %}{% endblock %}">
		{% block navigation %}
			{% include 'cruditor/includes/navigation.html' %}
		{% endblock %}

		<div class="container">
			{% block breadcrumb %}
				{% include 'cruditor/includes/breadcrumb.html' %}
			{% endblock %}

			{% block messages %}
				{% include 'cruditor/includes/messages.html' %}
			{% endblock %}

			{% block container %}
				{% block title %}
					<div class="cruditor-title row">
						<div class="col-auto mr-auto">
							<h2>{{ cruditor.title }}</h2>
						</div>
						<div class="col-auto">
							{% block title_buttons %}
								{% include 'cruditor/includes/titlebuttons.html' %}
							{% endblock %}
						</div>
					</div>
				{% endblock %}

				{% block content %}
				{% endblock %}
			{% endblock %}
		</div>

		{% block scripts %}
			<script src="{% static 'cruditor/js/cruditor.js' %}" async defer></script>
		{% endblock %}

		{% block extrabody %}
		{% endblock %}
	</body>
</html>
